@@ -46,6 +46,28 @@ class ASRBuilder {
         return ASRUtils::EXPR(ASR::make_Var_t(al, loc, sym));
     }
 
+    void VariableDeclaration(SymbolTable *symtab, std::string var_name,
+            ASR::ttype_t *type, ASR::intentType intent,
+            ASR::abiType abi=ASR::abiType::Source, bool a_value_attr=false) {
+        ASR::symbol_t* sym = ASR::down_cast<ASR::symbol_t>(
+            ASR::make_Variable_t(al, loc, symtab, s2c(al, var_name), nullptr, 0,
+            intent, nullptr, nullptr, ASR::storage_typeType::Default, type, nullptr, abi,
+            ASR::Public, ASR::presenceType::Required, a_value_attr));
+        symtab->add_symbol(s2c(al, var_name), sym);
+        return;
+    }
+
+    ASR::expr_t *VariableOverwrite(SymbolTable *symtab, std::string var_name,
+            ASR::ttype_t *type, ASR::intentType intent,
+            ASR::abiType abi=ASR::abiType::Source, bool a_value_attr=false) {
+        ASR::symbol_t* sym = ASR::down_cast<ASR::symbol_t>(
+            ASR::make_Variable_t(al, loc, symtab, s2c(al, var_name), nullptr, 0,
+            intent, nullptr, nullptr, ASR::storage_typeType::Default, type, nullptr, abi,
+            ASR::Public, ASR::presenceType::Required, a_value_attr));
+        symtab->add_or_overwrite_symbol(s2c(al, var_name), sym);
+        return ASRUtils::EXPR(ASR::make_Var_t(al, loc, sym));
+    }
+
     #define declare(var_name, type, intent)                                     \
         b.Variable(fn_symtab, var_name, type, ASR::intentType::intent)
 
@@ -111,11 +133,23 @@ class ASRBuilder {
         return make_Array_t_util(al, loc, type, m_dims.p, m_dims.n);
     }
 
+    ASR::ttype_t* CPtr() {
+        return TYPE(ASR::make_CPtr_t(al, loc));
+    }
+
     // Expressions -------------------------------------------------------------
     ASR::expr_t* Var(ASR::symbol_t* sym) {
         return ASRUtils::EXPR(ASR::make_Var_t(al, loc, sym));
     }
 
+    ASR::expr_t* ArrayUBound(ASR::expr_t* x, int64_t dim) {
+        return ASRUtils::EXPR(ASR::make_ArrayBound_t(al, loc, x, i32(dim), int32, ASR::arrayboundType::UBound, nullptr));
+    }
+
+    ASR::expr_t* ArrayLBound(ASR::expr_t* x, int64_t dim) {
+        return ASRUtils::EXPR(ASR::make_ArrayBound_t(al, loc, x, i32(dim), int32, ASR::arrayboundType::LBound, nullptr));
+    }
+
     inline ASR::expr_t* i_t(int64_t x, ASR::ttype_t* t) {
         return EXPR(ASR::make_IntegerConstant_t(al, loc, x, t));
     }
@@ -164,6 +198,24 @@ class ASRBuilder {
         return EXPR(ASR::make_ComplexConstant_t(al, loc, x, y, complex64));
     }
 
+    inline ASR::expr_t* constant_t(double x, ASR::ttype_t* t, double y = 0.0) {
+        if (ASRUtils::is_integer(*t)) {
+            return i_t(x, t);
+        } else if (ASRUtils::is_real(*t)) {
+            return f_t(x, t);
+        } else if (ASRUtils::is_complex(*t)) {
+            return complex_t(x, y, t);
+        } else if (ASRUtils::is_logical(*t)) {
+            if (x == 0.0) {
+                return bool_t(false, t);
+            } else {
+                return bool_t(true, t);
+            }
+        } else {
+            throw LCompilersException("Type not supported");
+        }
+    }
+
     inline ASR::expr_t* ListItem(ASR::expr_t* x, ASR::expr_t* pos, ASR::ttype_t* type) {
         return EXPR(ASR::make_ListItem_t(al, loc, x, pos, type, nullptr));
     }
@@ -201,6 +253,10 @@ class ASRBuilder {
             EXPR(ASR::make_StringConstant_t(al, loc, s2c(al, s), type)), t, nullptr));
     }
 
+    inline ASR::expr_t* PointerToCPtr(ASR::expr_t* x, ASR::ttype_t* t) {
+        return EXPR(ASR::make_PointerToCPtr_t(al, loc, x, t, nullptr));
+    }
+
     // Cast --------------------------------------------------------------------
 
     inline ASR::expr_t* r2i_t(ASR::expr_t* x, ASR::ttype_t* t) {
@@ -724,6 +780,10 @@ class ASRBuilder {
         return STMT(ASR::make_Assignment_t(al, loc, lhs, rhs, nullptr));
     }
 
+    ASR::stmt_t* CPtrToPointer(ASR::expr_t* cptr, ASR::expr_t* ptr, ASR::expr_t* shape = nullptr, ASR::expr_t* lower_bounds = nullptr) {
+        return STMT(ASR::make_CPtrToPointer_t(al, loc, cptr, ptr, shape, lower_bounds));
+    }
+
     template <typename T>
     ASR::stmt_t *Assign_Constant(ASR::expr_t *lhs, T init_value) {
         ASR::ttype_t *type = expr_type(lhs);
@@ -758,6 +818,21 @@ class ASRBuilder {
             nullptr, nullptr, nullptr));
     }
 
+    ASR::stmt_t *Allocate(ASR::expr_t *m_a, ASR::dimension_t* m_dims, size_t n_dims) {
+        Vec<ASR::alloc_arg_t> alloc_args; alloc_args.reserve(al, 1);
+        ASR::alloc_arg_t alloc_arg;
+        alloc_arg.loc = loc;
+        alloc_arg.m_a = m_a;
+        alloc_arg.m_dims = m_dims;
+        alloc_arg.n_dims = n_dims;
+        alloc_arg.m_type = nullptr;
+        alloc_arg.m_len_expr = nullptr;
+        alloc_args.push_back(al, alloc_arg);
+        return STMT(ASR::make_Allocate_t(al, loc, alloc_args.p, 1,
+            nullptr, nullptr, nullptr));
+    }
+
+
     #define UBound(arr, dim) PassUtils::get_bound(arr, dim, "ubound", al)
     #define LBound(arr, dim) PassUtils::get_bound(arr, dim, "lbound", al)
 
@@ -915,6 +990,23 @@ class ASRBuilder {
         return s;
     }
 
+    ASR::symbol_t* create_c_func_subroutines(std::string c_func_name, SymbolTable* fn_symtab, int n_args, ASR::ttype_t* arg_types) {
+        SymbolTable *fn_symtab_1 = al.make_new<SymbolTable>(fn_symtab);
+        Vec<ASR::expr_t*> args_1; args_1.reserve(al, 0);
+        for (int i = 0; i < n_args; i++) {
+            args_1.push_back(al, this->Variable(fn_symtab_1, "x_"+std::to_string(i), arg_types,
+                ASR::intentType::InOut, ASR::abiType::BindC, true));
+        }
+        ASR::expr_t *return_var_1 = this->Variable(fn_symtab_1, c_func_name,
+           ASRUtils::type_get_past_array(ASRUtils::type_get_past_allocatable(arg_types)),
+           ASRUtils::intent_return_var, ASR::abiType::BindC, false);
+        SetChar dep_1; dep_1.reserve(al, 1);
+        Vec<ASR::stmt_t*> body_1; body_1.reserve(al, 1);
+        ASR::symbol_t *s = make_ASR_Function_t(c_func_name, fn_symtab_1, dep_1, args_1,
+            body_1, return_var_1, ASR::abiType::BindC, ASR::deftypeType::Interface, s2c(al, c_func_name));
+        return s;
+    }
+
 };
 
 } // namespace LCompilers::ASRUtils
