@@ -196,38 +196,157 @@ public:
         pass_result.reserve(al, 1);
     }
 
+    /*
+    * Converts an array section expression like `array(1, :)` to an array item
+    * expression `array(1, var)` using the current `do` loop variable `var` and
+    * returns it. 
+    * 
+    * Returns the original expression if it is not `ASR::ArraySection_t`.
+    * 
+    * We do this conversion in the `where` pass before the `array_op` pass to use the
+    * current loop variable `var` for accessing values from the array. The `array_op` pass
+    * does the above conversion using a new `do` loop which leads to an incorrect output.
+    */
+    ASR::expr_t* make_array_item_from_array_section(ASR::expr_t* expression, ASR::expr_t* var) {
+        ASR::ArraySection_t* arr_section = nullptr;
+        if (ASR::is_a<ASR::ArraySection_t>(*expression)) {
+            arr_section = ASR::down_cast<ASR::ArraySection_t>(expression);
+        } else {
+            return expression;
+        }
+
+        ASR::expr_t* arr_section_var = arr_section->m_v;
+
+        size_t sliced_dim_index = 0;
+        for (size_t i = 0; i < arr_section->n_args; i++) {
+            if (!(arr_section->m_args[i].m_left == nullptr
+                  && arr_section->m_args[i].m_right != nullptr
+                  && arr_section->m_args[i].m_step == nullptr)) {
+                sliced_dim_index = i + 1;
+            }
+        }
+
+        Vec<ASR::array_index_t> args;
+        ASR::array_index_t ai;
+        ai.loc = arr_section_var->base.loc;
+        ai.m_left = nullptr;
+        ai.m_right = nullptr;
+        ai.m_step = nullptr;
+        args.reserve(al, 1);
+
+        ASR::expr_t* one = ASRUtils::EXPR(ASR::make_IntegerConstant_t(al, arr_section_var->base.loc, 1,
+                                ASRUtils::TYPE(ASR::make_Integer_t(al, arr_section_var->base.loc, 8))));
+        for (size_t i = 0; i < arr_section->n_args; i++) {
+            if (i + 1 == sliced_dim_index) {
+                ai.m_left = one;
+                ai.m_right = var;
+                ai.m_step = one;
+                args.push_back(al, ai);
+            } else {
+                args.push_back(al, arr_section->m_args[i]);
+            }
+        }
+
+        ASR::expr_t* array_item = ASRUtils::EXPR(
+                                    ASRUtils::make_ArrayItem_t_util(
+                                        al,
+                                        arr_section->m_v->base.loc,
+                                        arr_section->m_v,
+                                        args.p,
+                                        args.size(),
+                                        ASRUtils::type_get_past_array_pointer_allocatable(
+                                            ASRUtils::expr_type((arr_section_var))),
+                                        ASR::arraystorageType::ColMajor,
+                                        nullptr));
+
+        return array_item;
+    }
+
+    /*
+    * Converts an array section assignment statement like `array(1, :) = 2.0` into
+    * an array item assignment statement `array(1, var) = 2.0` using the current
+    * `do` loop variable `var`.
+    */
+    ASR::stmt_t* convert_array_section_assignment_to_array_item_assignment(ASR::Assignment_t* assignment,
+                                                                   ASR::expr_t* var) {
+        ASR::expr_t* target = make_array_item_from_array_section(assignment->m_target, var);
+
+        ASR::stmt_t* arr_item_assign = ASRUtils::STMT(
+                                        ASR::make_Assignment_t(al,
+                                            assignment->base.base.loc,
+                                            target,
+                                            assignment->m_value,
+                                            assignment->m_overloaded));
+
+        return arr_item_assign;
+    }
+
     ASR::stmt_t* handle_If(ASR::Where_t& x, ASR::expr_t* test, ASR::expr_t* var, Location& loc, Vec<ASR::expr_t*> idx_vars) {
         ASR::IntegerCompare_t* int_cmp = nullptr;
         ASR::RealCompare_t* real_cmp = nullptr;
+        ASR::LogicalBinOp_t* log_bin_op = nullptr;
         ASR::expr_t* left, *right;
+        bool is_left_array = false;
         bool is_right_array = false;
         ASR::ttype_t* logical_type = ASRUtils::TYPE(ASR::make_Logical_t(al, loc, 4));
+        ASR::expr_t* test_new = nullptr;
+        ASR::expr_t* left_array = nullptr;
+        ASR::expr_t* right_array = nullptr;
 
         if (ASR::is_a<ASR::IntegerCompare_t>(*test)) {
             int_cmp = ASR::down_cast<ASR::IntegerCompare_t>(test);
-            left = int_cmp->m_left;
-            right = int_cmp->m_right;
+
+            left = make_array_item_from_array_section(int_cmp->m_left, var);
+            right = make_array_item_from_array_section(int_cmp->m_right, var);
         } else if (ASR::is_a<ASR::RealCompare_t>(*test)) {
             real_cmp = ASR::down_cast<ASR::RealCompare_t>(test);
-            left = real_cmp->m_left;
-            right = real_cmp->m_right;
+
+            left = make_array_item_from_array_section(real_cmp->m_left, var);
+            right = make_array_item_from_array_section(real_cmp->m_right, var);
+        } else if (ASR::is_a<ASR::LogicalBinOp_t>(*test)) {
+            log_bin_op = ASR::down_cast<ASR::LogicalBinOp_t>(test);
+
+            left = make_array_item_from_array_section(log_bin_op->m_left, var);
+            right = make_array_item_from_array_section(log_bin_op->m_right, var);
         } else {
             throw LCompilersException("Unsupported type");
         }
 
-        if (ASRUtils::is_array(ASRUtils::expr_type(right))) {
-            is_right_array = true;
+        if (ASRUtils::is_array(ASRUtils::expr_type(left))) {
+            if (ASR::is_a<ASR::ArrayBroadcast_t>(*left)) {
+                ASR::ArrayBroadcast_t* arr_broadcast = ASR::down_cast<ASR::ArrayBroadcast_t>(left);
+                if (ASR::is_a<ASR::Logical_t>(*ASRUtils::expr_type(arr_broadcast->m_array))) {
+                    is_left_array = false;
+                }
+            } else {
+                is_left_array = true;
+                left_array = PassUtils::create_array_ref(left, idx_vars, al, current_scope);
+            }
         }
 
-        ASR::expr_t* left_array = PassUtils::create_array_ref(left, idx_vars, al, current_scope);
-        ASR::expr_t* right_array = PassUtils::create_array_ref(right, idx_vars, al, current_scope);
+        if (ASRUtils::is_array(ASRUtils::expr_type(right))) {
+            if (ASR::is_a<ASR::ArrayBroadcast_t>(*right)) {
+                ASR::ArrayBroadcast_t* arr_broadcast = ASR::down_cast<ASR::ArrayBroadcast_t>(right);
+                if (ASR::is_a<ASR::Logical_t>(*ASRUtils::expr_type(arr_broadcast->m_array))) {
+                    is_right_array = false;
+                }
+            } else {
+                is_right_array = true;
+                right_array = PassUtils::create_array_ref(right, idx_vars, al, current_scope);
+            }
+        }
 
-        ASR::expr_t* test_new = ASRUtils::EXPR(
-                    real_cmp?ASR::make_RealCompare_t(al, loc, left_array, real_cmp->m_op, is_right_array?right_array:right,
-                                            logical_type, nullptr):
-                             ASR::make_IntegerCompare_t(al, loc, left_array, int_cmp->m_op, is_right_array?right_array:right,
-                                            logical_type, nullptr));
 
+        if (int_cmp) {
+            test_new = ASRUtils::EXPR(ASR::make_IntegerCompare_t(al, loc, is_left_array ? left_array : left, int_cmp->m_op,
+                        is_right_array?right_array:right, logical_type, nullptr));
+        } else if (real_cmp) {
+            test_new = ASRUtils::EXPR(ASR::make_RealCompare_t(al, loc, is_left_array ? left_array : left, real_cmp->m_op,
+                        is_right_array?right_array:right, logical_type, nullptr));
+        } else if (log_bin_op) {
+            test_new = ASRUtils::EXPR(ASR::make_LogicalBinOp_t(al, loc, is_left_array ? left_array : left, log_bin_op->m_op,
+                        is_right_array?right_array:right, logical_type, nullptr));
+        }
 
         Vec<ASR::stmt_t*> if_body;
         if_body.reserve(al, x.n_body);
@@ -235,8 +354,13 @@ public:
             ASR::stmt_t* stmt = x.m_body[i];
             if (stmt->type == ASR::stmtType::Assignment) {
                 ASR::Assignment_t* assign_ = ASR::down_cast<ASR::Assignment_t>(stmt);
-                uint64_t h = get_hash((ASR::asr_t*) assign_);
-                assignment_hash[h] = idx_vars;
+                if (ASR::is_a<ASR::ArraySection_t>(*assign_->m_target)) {
+                   stmt = convert_array_section_assignment_to_array_item_assignment(assign_, var);
+                   pass_result.push_back(al, stmt);
+                } else {
+                    uint64_t h = get_hash((ASR::asr_t*) assign_);
+                    assignment_hash[h] = idx_vars;
+                }
             }
             if_body.push_back(al, stmt);
         }
@@ -252,8 +376,13 @@ public:
                 ASR::stmt_t* stmt = x.m_orelse[i];
                 if (stmt->type == ASR::stmtType::Assignment) {
                     ASR::Assignment_t* assign_ = ASR::down_cast<ASR::Assignment_t>(stmt);
-                    uint64_t h = get_hash((ASR::asr_t*) assign_);
-                    assignment_hash[h] = idx_vars;
+                    if (ASR::is_a<ASR::ArraySection_t>(*assign_->m_target)) {
+                        stmt = convert_array_section_assignment_to_array_item_assignment(assign_, var);
+                        pass_result.push_back(al, stmt);
+                    } else {
+                        uint64_t h = get_hash((ASR::asr_t*) assign_);
+                        assignment_hash[h] = idx_vars;
+                    }
                 }
                 orelse_body.push_back(al, stmt);
             }
@@ -262,15 +391,45 @@ public:
         return if_stmt;
     }
 
+    ASR::stmt_t* nested_do_loop(Location &loc,ASR::expr_t* left_array,Vec<ASR::expr_t*> &idx_vars,Vec<ASR::stmt_t*> &do_loop_body_deep,int current_idx=0){
+        // Base Case.
+        if(current_idx == (int)idx_vars.size()){
+            return nullptr;
+        }
+
+        // Create Head.
+        ASR::do_loop_head_t head;
+        head.loc = loc;
+        head.m_v = idx_vars[current_idx];
+        head.m_start = PassUtils::get_bound(left_array, current_idx+1, "lbound", al);
+        head.m_end = PassUtils::get_bound(left_array, current_idx+1, "ubound", al);
+        ASR::ttype_t* int32_type = ASRUtils::TYPE(ASR::make_Integer_t(al, loc, 4));
+        head.m_increment = ASRUtils::EXPR(ASR::make_IntegerConstant_t(al, loc, 1, int32_type));
+
+        // Create Body
+        ASR::stmt_t* nested = nested_do_loop(loc, left_array, idx_vars, do_loop_body_deep, current_idx+1); 
+        Vec<ASR::stmt_t*> do_loop_body;
+        do_loop_body.reserve(al, 1);
+        if(!nested){
+            // Use the do_loop_body initialized in the caller function when we hit last idx_var.
+            do_loop_body = do_loop_body_deep; 
+        } else {
+            do_loop_body.push_back(al, nested);
+        }
+
+        return ASRUtils::STMT(ASR::make_DoLoop_t(al, loc, nullptr, head, do_loop_body.p, do_loop_body.size(), nullptr, 0));
+    }
     void visit_Where(const ASR::Where_t& x) {
         ASR::Where_t& xx = const_cast<ASR::Where_t&>(x);
         Location loc = x.base.base.loc;
         ASR::expr_t* test = x.m_test;
         ASR::IntegerCompare_t* int_cmp = nullptr;
         ASR::RealCompare_t* real_cmp = nullptr;
+        ASR::LogicalBinOp_t* log_bin_op = nullptr;
         ASR::expr_t* left;
         ASR::expr_t* opt_left = nullptr;
         ASR::stmt_t* assign_stmt = nullptr;
+        ASR::stmt_t* if_stmt = nullptr;
 
         if (ASR::is_a<ASR::IntegerCompare_t>(*test)) {
             int_cmp = ASR::down_cast<ASR::IntegerCompare_t>(test);
@@ -278,20 +437,18 @@ public:
         } else if (ASR::is_a<ASR::RealCompare_t>(*test)) {
             real_cmp = ASR::down_cast<ASR::RealCompare_t>(test);
             left = real_cmp->m_left;
+        } else if (ASR::is_a<ASR::LogicalBinOp_t>(*test)) {
+            log_bin_op = ASR::down_cast<ASR::LogicalBinOp_t>(test);
+            left = log_bin_op->m_left;
         } else {
             throw LCompilersException("Unsupported type, " + std::to_string(test->type));
         }
 
-        // Construct a do loop
-        ASR::stmt_t* doloop = nullptr;
-
-        // create a index variable
+        // create index variables.
         Vec<ASR::expr_t*> idx_vars;
-        PassUtils::create_idx_vars(idx_vars, 1, loc, al, current_scope);
+        PassUtils::create_idx_vars(idx_vars, ASRUtils::extract_n_dims_from_ttype(ASRUtils::expr_type(left)), loc, al, current_scope);
         ASR::expr_t* var = idx_vars[0];
 
-        ASR::ttype_t* int32_type = ASRUtils::TYPE(ASR::make_Integer_t(al, loc, 4));
-
         if (ASR::is_a<ASR::FunctionCall_t>(*left)) {
             // Create an assignment `return_var = left` and replace function call with return_var
             ASR::FunctionCall_t* fc = ASR::down_cast<ASR::FunctionCall_t>(left);
@@ -315,27 +472,31 @@ public:
             real_cmp = ASR::down_cast<ASR::RealCompare_t>(test);
             real_cmp->m_left = opt_left;
         }
+        if (opt_left && ASR::is_a<ASR::LogicalBinOp_t>(*test)) {
+            log_bin_op = ASR::down_cast<ASR::LogicalBinOp_t>(test);
+            log_bin_op->m_left = opt_left;
+        }
 
-        // create do loop head
-        ASR::do_loop_head_t head;
-        head.loc = loc;
-        head.m_v = var;
-        head.m_start = PassUtils::get_bound(opt_left?opt_left:left, 1, "lbound", al);
-        head.m_end = PassUtils::get_bound(opt_left?opt_left:left, 1, "ubound", al);
-        head.m_increment = ASRUtils::EXPR(ASR::make_IntegerConstant_t(al, loc, 1, int32_type));
-
-        // create do loop body
+        //Create do loop.
+        ASR::stmt_t* doloop = nullptr;
         Vec<ASR::stmt_t*> do_loop_body;
         do_loop_body.reserve(al, 1);
 
         // create an if statement
-        ASR::stmt_t* if_stmt = handle_If(xx, int_cmp?ASRUtils::EXPR((ASR::asr_t*)int_cmp):ASRUtils::EXPR((ASR::asr_t*)real_cmp), var, loc, idx_vars);
+        // TO DO : fix handle_if function to handle arraySections properly while doing looping on multiple dimensions.
+        if (int_cmp) {
+            if_stmt = handle_If(xx, ASRUtils::EXPR((ASR::asr_t*)int_cmp), var, loc, idx_vars);
+        } else if (real_cmp) {
+            if_stmt = handle_If(xx, ASRUtils::EXPR((ASR::asr_t*)real_cmp), var, loc, idx_vars);
+        } else if (log_bin_op) {
+            if_stmt = handle_If(xx, ASRUtils::EXPR((ASR::asr_t*)log_bin_op), var, loc, idx_vars);
+        }
         if (assign_stmt) {
             pass_result.push_back(al, assign_stmt);
         }
-        do_loop_body.push_back(al, if_stmt);
 
-        doloop = ASRUtils::STMT(ASR::make_DoLoop_t(al, loc, 0, head, do_loop_body.p, do_loop_body.size(), nullptr, 0));
+        do_loop_body.push_back(al, if_stmt);
+        doloop = nested_do_loop(loc, opt_left?opt_left:left, idx_vars, do_loop_body);
         pass_result.push_back(al, doloop);
     }
 };
